
-- Script made by Oysi

local scene = {}
for i = 1, 100 do
	scene[i] = {
		a = {x = -1, y = -1, z = -i};
		b = {x =  0, y =  1, z = -i};
		c = {x =  1, y = -1, z = -i};
		color = {math.random()*255, math.random()*255, math.random()*255};
	}
end

function project(vertex)
	local x = vertex.x / -vertex.z
	local y = vertex.y / -vertex.z * -1
	
	if vertex.z > 0 then
		x = x * -10
		y = y * -10
	end
	
	x = 400 + x*400
	y = 300 + y*400
	
	return x, y, vertex.z
end

function love.draw()
	local render = {}
	
	for _, poly in pairs(scene) do
		local ax, ay, az = project(poly.a)
		local bx, by, bz = project(poly.b)
		local cx, cy, cz = project(poly.c)
		
		if az < 0 or bz < 0 or cz < 0 then
			render[#render + 1] = {
				vertices = {
					ax, ay;
					bx, by;
					cx, cy;
				};
				poly = poly;
				z = (az + bz + cz)/3;
			}
		end
	end
	
	table.sort(render, function(a, b) return a.z < b.z end)
	
	for _, data in pairs(render) do
		local poly = data.poly
		
		love.graphics.setColor(poly.color)
		love.graphics.polygon("fill", data.vertices)
	end
end

function translate(x, y, z)
	for _, poly in pairs(scene) do
		for _, vertex in pairs {poly.a, poly.b, poly.c} do
			vertex.x = vertex.x + x
			vertex.y = vertex.y + y
			vertex.z = vertex.z + z
		end
	end
end

function rotateY(r)
	local cosR = math.cos(r)
	local sinR = math.sin(r)
	for _, poly in pairs(scene) do
		for _, vertex in pairs {poly.a, poly.b, poly.c} do
			local x = vertex.x
			local z = vertex.z
			vertex.x = x*cosR + z*sinR
			vertex.z = x*-sinR + z*cosR
		end
	end
end

function love.update(delta)
	local moveSpeed = delta
	local rotSpeed = delta
	
	if love.keyboard.isDown("a") then
		translate(moveSpeed, 0, 0)
	end
	if love.keyboard.isDown("d") then
		translate(-moveSpeed, 0, 0)
	end
	
	if love.keyboard.isDown("w") then
		translate(0, 0, moveSpeed)
	end
	if love.keyboard.isDown("s") then
		translate(0, 0, -moveSpeed)
	end
	
	if love.keyboard.isDown("q") then
		rotateY(-rotSpeed)
	end
	if love.keyboard.isDown("e") then
		rotateY(rotSpeed)
	end
end
